#========================================================================================
# package      DE_EPAGES::AmazonPayments::t::unit::PaymentMethodAmazonPayments
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments
#========================================================================================
package DE_EPAGES::AmazonPayments::t::unit::PaymentMethodAmazonPayments;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_callbackUrl_https();
    $self->test_callbackUrl_http();
    $self->test_callbackUrl_https_and_undefinedWebServerScriptNamePart();

    done_testing();
}



sub test_callbackUrl_https {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ProtocolAndServerSSL = 'https://myShop.test';
    my $WebServerScriptNamePart = 'MyShop';
    my $PaymentMethod = $self->_createPaymentMethodAmazonPayments($ProtocolAndServerSSL, $WebServerScriptNamePart);

    my $Url = $PaymentMethod->callbackUrl();

    is($Url, 'http://test.org/callback', "$SubTestName - test if callback url is returned");
}



sub test_callbackUrl_http {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ProtocolAndServerSSL = 'http://myShop.test';
    my $WebServerScriptNamePart = 'MyShop';
    my $PaymentMethod = $self->_createPaymentMethodAmazonPayments($ProtocolAndServerSSL, $WebServerScriptNamePart);

    throws_ok(sub {
        $PaymentMethod->callbackUrl();
    }, qr/NoValidSSLCertificateError/, "$SubTestName - test if exception was thrown.");
}



sub test_callbackUrl_https_and_undefinedWebServerScriptNamePart {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ProtocolAndServerSSL = 'https://myShop.test';
    my $WebServerScriptNamePart = undef;
    my $PaymentMethod = $self->_createPaymentMethodAmazonPayments($ProtocolAndServerSSL, $WebServerScriptNamePart);

    throws_ok(sub {
        $PaymentMethod->callbackUrl();
    }, qr/NoWebServerScriptNamePartError/, "$SubTestName - test if exception was thrown.");

}



sub _createPaymentMethodAmazonPayments {
    my $self = shift;
    my ($ProtocolAndServerSSL, $WebServerScriptNamePart) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
       'DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments',
       []
    );
    $Mockify->addMockWithReturnValue('getSite', $self->_createSite($ProtocolAndServerSSL, $WebServerScriptNamePart));
    $Mockify->addMockWithReturnValueAndParameterCheck('_generateUrl', 'http://test.org/callback', ['hashref']);
    return $Mockify->getMockObject();
}



sub _createSite {
    my $self = shift;
    my ($ProtocolAndServerSSL, $WebServerScriptNamePart) = @_;
    my $hAttributes = {
        'Alias' => 'MyShop',
        'PathString' => '/test/path',
        'ProtocolAndServerSSL' => $ProtocolAndServerSSL,
        'WebServerScriptNamePart' => $WebServerScriptNamePart,
    };
    my $MockedSite = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );
    return $MockedSite;
}

__PACKAGE__->runScript();