#========================================================================================
# package      DE_EPAGES::Order::t::unit::PickupTokenExpiration
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::PickupTokenExpiration
#========================================================================================
package DE_EPAGES::Order::t::unit::PickupTokenExpiration;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::PickupTokenExpiration;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_PickupTokenExpiration_NotExpired();
    $self->test_PickupTokenExpiration_Expired();

    done_testing();
}




sub test_PickupTokenExpiration_NotExpired {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PickupTokenExpiration = $self->_createPickupTokenExpiration();

    my $BasketCreationDate = $self->_createDateTime(30); ## no critic (ProhibitMagicNumbers)
    my $IsExpired = $PickupTokenExpiration->isExpired($BasketCreationDate);

    isnt($IsExpired, 1,"$SubTestName - test if basket is not expired");
}



sub test_PickupTokenExpiration_Expired {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PickupTokenExpiration = $self->_createPickupTokenExpiration();

    my $BasketCreationDate = $self->_createDateTime(5);
    my $IsExpired = $PickupTokenExpiration->isExpired($BasketCreationDate);

    is($IsExpired, 1,"$SubTestName - test if basket is expired");
}



sub _createPickupTokenExpiration {
    my $self = shift;
    my ($IsBasketExpired) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::PickupTokenExpiration',
         []
    );
    my $CurrentTimeMinute = 36; ## no critic (ProhibitMagicNumbers)
    $Mockify->addMockWithReturnValue('_getCurrentTime', $self->_createDateTime($CurrentTimeMinute));
    $Mockify->addMockWithReturnValue('_GetLocalTimeZone', 'Europe/Berlin');
    $Mockify->addMockWithReturnValue('_LoadRootObject', $self->_createSystem());
    return $Mockify->getMockObject();
}



sub _createSystem {
    my $self = shift;
    my $hAtributes = {
        'SessionTimeout' => '20',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site', $hAtributes);
}



sub _createDateTime {
    my $self = shift;
    my ($Minute) = @_;
    my $DateTime = DateTime->new(
        year       => 2015,
        month      => 4,
        day        => 21,
        hour       => 1,
        minute     => $Minute,
        second     => 3,
        nanosecond => 0,
        time_zone  => 'Europe/Berlin',
    );
    return $DateTime;
}

__PACKAGE__->runScript();