#========================================================================================
# package      DE_EPAGES::Order::t::unit::PickupTokenValidation
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::PickupTokenValidation
#========================================================================================
package DE_EPAGES::Order::t::unit::PickupTokenValidation;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::PickupTokenValidation;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Test::Mock::Mockify;
use HTTP::Request;
use Test::MockObject::Extends;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_getUrlForPickupTokenValidation();
    $self->test_checkPickupToken_validToken();
    $self->test_checkPickupToken_invalidToken();

    done_testing();
}



sub test_getUrlForPickupTokenValidation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PickupTokenValidation = $self->_createPickupTokenValidation();
    my $Servlet = $self->_createPickupBasketRequestServlet();
    $Servlet->vars('HTTP_HOST', 'test.org');

    my $Url = $PickupTokenValidation->getUrlForPickupTokenValidation($Servlet);

    is($Url, 'http://test.org/pickupBasket?test=1', "$SubTestName - test if token check returns positive result");
}



sub test_checkPickupToken_validToken {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PickupTokenValidation = $self->_createPickupTokenValidation();
    $PickupTokenValidation->mock('_ValidateDereferrerToken', sub {
        return 1;
    });

    my $IsTokenValid = $PickupTokenValidation->checkPickupToken('133', 'http://test.org');
    ok($IsTokenValid, "$SubTestName - test if token check returns positive result");
}



sub test_checkPickupToken_invalidToken {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PickupTokenValidation = $self->_createPickupTokenValidation();
    $PickupTokenValidation->mock('_ValidateDereferrerToken', sub {
        Error('INVALID_DEREFERRER_TOKEN');
    });

    my $IsTokenValid = $PickupTokenValidation->checkPickupToken('133', 'http://test.org');
    is($IsTokenValid, 0, "$SubTestName - test if token check returns negative result");
}



sub _createPickupTokenValidation {
    my $self = shift;
    my $PickupTokenValidation = DE_EPAGES::Order::API::PickupTokenValidation->new();
    my $MockedPickupTokenValidation = Test::MockObject::Extends->new( $PickupTokenValidation );
    return $MockedPickupTokenValidation;
}


sub _createPickupBasketRequestServlet {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Presentation::Servlet::Permission',
         []
    );
    $Mockify->addMockWithReturnValue('request', $self->_createRequest());
    return $Mockify->getMockObject();
}



sub _createRequest {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'HTTP::Request',
         []
    );
    $Mockify->addMockWithReturnValue('uri', '/pickupBasket?test=1&PickupToken=12345');
    my $MockedRequest = $Mockify->getMockObject();
    $MockedRequest->mock('header', sub {return 0});
    return $MockedRequest;
}

__PACKAGE__->runScript();