#========================================================================================
# package      DE_EPAGES::Order::t::unit::UI_Basket_Ext
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::UI::Basket_Ext
#========================================================================================
package DE_EPAGES::Order::t::unit::UI_Basket_Ext;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::UI::Basket_Ext;
use DE_EPAGES::Test::Mock::Mockify qw (WasCalled);
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;
    $self->test_PickupBasket_CorrectPickupToken();
    $self->test_PickupBasket_WrongPickupToken();
    $self->test_PickupBasket_BasketTooOld();
    done_testing();
}



sub test_PickupBasket_CorrectPickupToken {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Servlet = $self->_createServlet();
    $Servlet->params('PickupToken', 'TestPickupToken');
    my $IsBasketExpired = 0;
    my $IsTokenValid = 1;
    my $Controller = $self->_create_Basket_Ext_Controller($IsBasketExpired, $IsTokenValid);

    lives_ok(sub {
        $Controller->PickupBasket($Servlet)
    }, "$SubTestName - test if PickupBasket runs without exception");

    ok(WasCalled($Servlet, 'registerSession'),"$SubTestName - selftest");
}



sub test_PickupBasket_WrongPickupToken {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Servlet = $self->_createServlet();
    $Servlet->params('PickupToken', 'AnotherToken');
    my $IsBasketExpired = 0;
    my $IsTokenValid = 0;
    my $Controller = $self->_create_Basket_Ext_Controller($IsBasketExpired, $IsTokenValid);

    lives_ok(sub {
        $Controller->PickupBasket($Servlet)
    }, "$SubTestName - test if PickupBasket runs without exception");

    is(WasCalled($Servlet, 'registerSession'), 0,"$SubTestName - selftest");
}



sub test_PickupBasket_BasketTooOld {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Servlet = $self->_createServlet();
    $Servlet->params('PickupToken', 'TestPickupToken');
    my $IsBasketExpired = 1;
    my $IsTokenValid = 1;
    my $Controller = $self->_create_Basket_Ext_Controller($IsBasketExpired, $IsTokenValid);

    lives_ok(sub {
        $Controller->PickupBasket($Servlet)
    }, "$SubTestName - test if PickupBasket runs without exception");

    is(WasCalled($Servlet, 'registerSession'), 0,"$SubTestName - selftest");
}



sub _create_Basket_Ext_Controller {
    my $self = shift;
    my ($IsBasketExpired, $IsTokenValid) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::UI::Basket_Ext',
         []
    );
    $Mockify->addMockWithReturnValue('_createPickupTokenExpiration', $self->_createPickupTokenExpiration($IsBasketExpired));
    $Mockify->addMockWithReturnValue('_createPickupTokenValidation', $self->_createPickupTokenValidation($IsTokenValid));
    return $Mockify->getMockObject();
}



sub _createServlet {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Presentation::Servlet::Permission',
         []
    );
    $Mockify->addMockWithReturnValue('object', $self->_createBasket());
    $Mockify->addMockWithReturnValueAndParameterCheck('registerSession', undef, [{'object' => 'DE_EPAGES::Permission::API::Object::User'}]);
    return $Mockify->getMockObject();
}



sub _createPickupTokenExpiration {
    my $self = shift;
    my ($IsBasketExpired) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::PickupTokenExpiration',
         []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('isExpired', $IsBasketExpired, ['string']);
    return $Mockify->getMockObject();
}



sub _createPickupTokenValidation {
    my $self = shift;
    my ($IsTokenValid) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::PickupTokenValidation',
         []
    );
    my $Url = 'http://www.test.de/pickupBasket';
    $Mockify->addMockWithReturnValueAndParameterCheck('getUrlForPickupTokenValidation', $Url, ['object']);
    $Mockify->addMockWithReturnValueAndParameterCheck('checkPickupToken', $IsTokenValid, ['string', {'string' => 'http://www.test.de/pickupBasket'}]);
    return $Mockify->getMockObject();
}



sub _createBasket {
    my $self = shift;
    my $hAtributes = {
        'User' => $self->_createUser(),
        'PickupToken' => '', # database field is not used anymore
        'CreationDate' => 'Aug 21, 2015 3:03:57 PM',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAtributes);
}



sub _createUser {
    my $self = shift;
    my $hAtributes = {};
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Permission::API::Object::User', $hAtributes);
}

__PACKAGE__->runScript();